﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Diagnostics;
using System.Threading;


namespace KinectProject
{
    public class CarController
    {
        //constants
        private static int storedCarStataus = (int)Car.Forward; //Change it to STOP
        const int FORWARD = 0;
        const int RIGHT = 1;
        const int BACKWARD = 2;
        const int LEFT = 3;

        //com port
        private static SerialPort comPort = new SerialPort();

        public static void MoveCar(Car direction)
        {
            switch ((int)direction)
            {
                case (int)Car.Forward: //move forward

                    if (storedCarStataus != (int)Car.Forward)
                    {
                        AllPowerOff();
                        PowerOn(FORWARD);
                        storedCarStataus = (int)Car.Forward;
                    }
                    
                    break;

                case (int)Car.ForwardRight: // turn forward right
                    if (storedCarStataus != (int)Car.ForwardRight)
                    {
                        AllPowerOff();
                        PowerOn(FORWARD);
                        PowerOn(RIGHT);
                    }
                    storedCarStataus = (int)Car.ForwardRight;
                    break;

                case (int)Car.ForwardLeft: // turn forward left

                    if (storedCarStataus != (int)Car.ForwardLeft)
                    {
                        AllPowerOff();
                        PowerOn(FORWARD);
                        PowerOn(LEFT);
                    }
                    storedCarStataus = (int)Car.ForwardLeft;
                    break;
                
                case (int)Car.Backward: //move backward
                    if (storedCarStataus != (int)Car.Backward)
                    {
                        AllPowerOff();
                        PowerOn(BACKWARD);
                    }
                    storedCarStataus = (int)Car.Backward;
                    break;

                case (int)Car.BackwardRight: // turn backward right

                    if (storedCarStataus != (int)Car.BackwardRight)
                    {
                        AllPowerOff();
                        PowerOn(BACKWARD);
                        PowerOn(RIGHT);
                    }
                    storedCarStataus = (int)Car.BackwardRight;
                    break;

                case (int)Car.BackwardLeft: // turn backward left

                    if (storedCarStataus != (int)Car.BackwardLeft)
                    {
                        AllPowerOff();
                        PowerOn(BACKWARD);
                        PowerOn(LEFT);
                    }
                    storedCarStataus = (int)Car.BackwardLeft;
                    break;
                //Add Stop Case here
                default:
                    StopCar();
                    break;
            }
        }

        public static void StopCar()
        {
            storedCarStataus = (int)Car.Forward; //Change it to STOP
            AllPowerOff();
        }

        public static bool OpenPort(int portNo)
        {
            if (!comPort.IsOpen)
            {
                comPort.PortName = "COM" + portNo;
                try
                {
                    comPort.Open();
                    comPort.DiscardInBuffer();
                    comPort.DiscardOutBuffer();
                    Debug.WriteLine("CarController OpenPort: " + portNo);
                    return true;
                }
                catch
                {
                    return false;
                }

            }
            return false;
        }

        public static void ClosePort()
        {

            if ((comPort.IsOpen))
            {
                comPort.Close();
                Debug.WriteLine("CarController ClosePort");
            }
        }

        private static bool sendData(string data)
        {
            if (comPort.IsOpen)
            {
                comPort.DiscardOutBuffer();
                comPort.Write(data);
                return true;
            }
            return false;
        }

        private static bool PowerOn(int GpioNum)
        {
            return sendData("gpio set " + GpioNum.ToString() + "\r");
        }

        private static bool PowerOff(int GpioNum)
        {
            return sendData("gpio clear " + GpioNum.ToString() + "\r");
        }

        public static void AllPowerOff()
        {
            PowerOff(FORWARD);
            PowerOff(RIGHT);
            PowerOff(BACKWARD);
            PowerOff(LEFT);
        }
    }
}
